import { isEnumValue } from './enum.js';
import { isValidGraphqlName } from './name.js';
import { isRecord } from './record.js';
import { isVariablePlaceholder } from './variable-placeholder.js';
function isObjectValue(value) {
    return isRecord(value) && !isEnumValue(value) && !isVariablePlaceholder(value);
}
function isPrimitive(value) {
    const type = typeof value;
    return value === null || type === 'number' || type === 'string' || type === 'boolean';
}
function serializePrimitiveValue(value) {
    return JSON.stringify(value);
}
function normalizeObjectValue(value) {
    let referencedVariables = new Set();
    const serializedFields = [];
    for (const [fieldName, fieldValue] of Object.entries(value)) {
        if (!isValidGraphqlName(fieldName)) {
            throw new Error(`Field name "${fieldName}" is not a valid GraphQL field name`);
        }
        // eslint-disable-next-line @typescript-eslint/no-use-before-define -- mutual recursion
        const normalizedFieldValue = normalizeGraphqlValue(fieldValue);
        serializedFields.push(`${fieldName}: ${normalizedFieldValue.serializedValue}`);
        referencedVariables = new Set([...referencedVariables, ...normalizedFieldValue.referencedVariables]);
    }
    return {
        serializedValue: `{${serializedFields.join(', ')}}`,
        referencedVariables
    };
}
function normalizeListValue(value) {
    let referencedVariables = new Set();
    const serializedItems = [];
    for (const item of value) {
        // eslint-disable-next-line @typescript-eslint/no-use-before-define -- mutual recursion
        const normalizedItem = normalizeGraphqlValue(item);
        serializedItems.push(normalizedItem.serializedValue);
        referencedVariables = new Set([...referencedVariables, ...normalizedItem.referencedVariables]);
    }
    return {
        serializedValue: `[${serializedItems.join(', ')}]`,
        referencedVariables
    };
}
export function normalizeGraphqlValue(value) {
    const referencedVariables = new Set();
    if (isPrimitive(value)) {
        return {
            serializedValue: serializePrimitiveValue(value),
            referencedVariables
        };
    }
    if (isEnumValue(value)) {
        return {
            serializedValue: value.enumValue,
            referencedVariables
        };
    }
    if (isVariablePlaceholder(value)) {
        return {
            serializedValue: value.variableName,
            referencedVariables: new Set([...referencedVariables, value.variableName])
        };
    }
    if (isObjectValue(value)) {
        return normalizeObjectValue(value);
    }
    return normalizeListValue(value);
}
//# sourceMappingURL=value.js.map